// Every instrument must implement the Instrument interface so 
// playNote() can call the instrument's methods.
class ToneInstrument implements Instrument
{
  // create all variables that must be used througout the class
  Oscil sineOsc;
  ADSR  adsr;
  AudioOutput out;
  float attack;
  float freq;
  float amp;
  
  // constructor for this instrument
  ToneInstrument(AudioOutput output )
  {
    // equate class variables to constructor variables as necessary
    out = output;
    freq = random(50,1000);
    amp = random(0.01,0.3);
    // create new instances of any UGen objects as necessary
    sineOsc = new Oscil(freq, amp, Waves.TRIANGLE );
  
    // ADSR(maxamp, attack, decay, sustain, release
    adsr = new ADSR(0.5, 0.01, 0.01, 0.01, 0.01);
    
    // patch everything together up to the final output
    sineOsc.patch( adsr );
  }
  
  
  // every instrument must have a noteOn( float ) method
  void noteOn( float dur )
  {
    // turn on the ADSR
    adsr.noteOn();
    // patch to the output
    adsr.patch( out );
   }
  
  // every instrument must have a noteOff() method
  void noteOff()
  {
    // tell the ADSR to unpatch after the release is finished
    adsr.unpatchAfterRelease( out );
    // call the noteOff 
    adsr.noteOff();
  }
}
